@@ -37,6 +37,7 @@ import errno
 import time
 import traceback
 import base64
+import sys
 import locale
 
 try:
@@ -83,6 +84,15 @@ def get_proxy_data_from_dict(proxy):
         proxy_user, proxy_pass = proxy['user'], proxy['pass']
     return tcp_host, tcp_port, proxy_user, proxy_pass
 
+def decode_py2(string, encoding):
+    # decodes string into unicode if in py2
+    # py3 has unicode strings by default
+    try:
+        string = string.decode(encoding)
+    except AttributeError:
+        pass
+    return string
+
 #: timeout to connect to the server socket, it doesn't include auth
 CONNECT_TIMEOUT_SECONDS = 30
 
@@ -370,8 +380,8 @@ class NonBlockingTCP(NonBlockingTransport, IdleObject):
             self._sock.setblocking(False)
             self._sock.connect((self.server, self.port))
         except Exception as exc:
-            errnum, errstr = exc.errno,\
-                exc.strerror.decode(locale.getpreferredencoding())
+            errnum, errstr = exc.errno, \
+                    decode_py2(exc.strerror, locale.getpreferredencoding())
 
         if errnum in (errno.EINPROGRESS, errno.EALREADY, errno.EWOULDBLOCK):
             # connecting in progress
@@ -474,7 +484,7 @@ class NonBlockingTCP(NonBlockingTransport, IdleObject):
             self._sock.shutdown(socket.SHUT_RDWR)
             self._sock.close()
         except socket.error as e:
-            errstr = e.strerror.decode(locale.getpreferredencoding())
+            errstr = decode_py2(e.strerror, locale.getpreferredencoding())
             log.info('Error while disconnecting socket: %s' % errstr)
         self.fd = -1
         NonBlockingTransport.disconnect(self, do_callback)
@@ -558,7 +568,7 @@ class NonBlockingTCP(NonBlockingTransport, IdleObject):
                 return None
             self.sendbuff = self.sendqueue.pop(0)
         try:
-            send_count = self._send(self.sendbuff.encode("utf-8"))
+            send_count = self._send(self.sendbuff)
             if send_count:
                 sent_data = self.sendbuff[:send_count]
                 self.sendbuff = self.sendbuff[send_count:]
@@ -569,7 +579,7 @@ class NonBlockingTCP(NonBlockingTransport, IdleObject):
                     self.sent_bytes_buff = b''
                 # try to decode sent data
                 try:
-                    sent_data = sent_data.decode('utf-8')
+                    sent_data = decode_py2(sent_data, 'utf-8')
                 except UnicodeDecodeError:
                     for i in range(-1, -4, -1):
                         char = sent_data[i]
@@ -577,7 +587,7 @@ class NonBlockingTCP(NonBlockingTransport, IdleObject):
                             self.sent_bytes_buff = sent_data[i:]
                             sent_data = sent_data[:i]
                             break
-                    sent_data = sent_data.decode('utf-8')
+                    sent_data = decode_py2(sent_data, 'utf-8')
                 self.raise_event(DATA_SENT, sent_data)
 
         except Exception:
@@ -603,7 +613,7 @@ class NonBlockingTCP(NonBlockingTransport, IdleObject):
             log.info("_do_receive, caught SSL error, got %s:" % received,
                     exc_info=True)
             errnum, errstr = e.errno,\
-                e.strerror.decode(locale.getpreferredencoding())
+                decode_py2(e.strerror, locale.getpreferredencoding())
 
         if received == '':
             errstr = 'zero bytes on recv'
@@ -639,15 +649,17 @@ class NonBlockingTCP(NonBlockingTransport, IdleObject):
 
         # try to decode data
         try:
-            received = received.decode('utf-8')
+            received = decode_py2(received, 'utf-8')
         except UnicodeDecodeError:
             for i in range(-1, -4, -1):
                 char = received[i]
-                if ord(char) & 0xc0 == 0xc0:
+                if sys.version_info[0] < 3: # with py2 we get a str
+                    char = ord(char)
+                if char & 0xc0 == 0xc0:
                     self.received_bytes_buff = received[i:]
                     received = received[:i]
                     break
-            received = received.decode('utf-8')
+            received = decode_py2(received, 'utf-8')
 
         # pass received data to owner
         if self.on_receive:
@@ -766,9 +778,7 @@ class NonBlockingHTTP(NonBlockingTCP):
         Builds http message with given body. Values for headers and status line
         fields are taken from class variables
         """
-        absolute_uri = '%s://%s:%s%s' % (self.http_protocol, self.http_host,
-                self.http_port, self.http_path)
-        headers = ['%s %s %s' % (method, absolute_uri, self.http_version),
+        headers = ['%s %s %s' % (method, self.http_path, self.http_version),
                 'Host: %s:%s' % (self.http_host, self.http_port),
                 'User-Agent: Gajim',
                 'Content-Type: text/xml; charset=utf-8',
